@@ -33,18 +33,21 @@ static struct job2str {
   Id job;
   const char *str;
 } job2str[] = {
-  { SOLVER_NOOP,          "noop" },
-  { SOLVER_INSTALL,       "install" },
-  { SOLVER_ERASE,         "erase" },
-  { SOLVER_UPDATE,        "update" },
-  { SOLVER_WEAKENDEPS,    "weakendeps" },
-  { SOLVER_MULTIVERSION,  "multiversion" },
-  { SOLVER_MULTIVERSION,  "noobsoletes" },	/* old name */
-  { SOLVER_LOCK,          "lock" },
-  { SOLVER_DISTUPGRADE,   "distupgrade" },
-  { SOLVER_VERIFY,        "verify" },
-  { SOLVER_DROP_ORPHANED, "droporphaned" },
-  { SOLVER_USERINSTALLED, "userinstalled" },
+  { SOLVER_NOOP,           "noop" },
+  { SOLVER_INSTALL,        "install" },
+  { SOLVER_ERASE,          "erase" },
+  { SOLVER_UPDATE,         "update" },
+  { SOLVER_WEAKENDEPS,     "weakendeps" },
+  { SOLVER_MULTIVERSION,   "multiversion" },
+  { SOLVER_MULTIVERSION,   "noobsoletes" },	/* old name */
+  { SOLVER_LOCK,           "lock" },
+  { SOLVER_DISTUPGRADE,    "distupgrade" },
+  { SOLVER_VERIFY,         "verify" },
+  { SOLVER_DROP_ORPHANED,  "droporphaned" },
+  { SOLVER_USERINSTALLED,  "userinstalled" },
+  { SOLVER_ALLOWUNINSTALL, "allowuninstall" },
+  { SOLVER_FAVOR,          "favor" },
+  { SOLVER_DISFAVOR,       "disfavor" },
   { 0, 0 }
 };
 
@@ -77,6 +80,10 @@ static struct resultflags2str {
   { TESTCASE_RESULT_ORPHANED,		"orphaned" },
   { TESTCASE_RESULT_RECOMMENDED,	"recommended" },
   { TESTCASE_RESULT_UNNEEDED,		"unneeded" },
+  { TESTCASE_RESULT_ALTERNATIVES,	"alternatives" },
+  { TESTCASE_RESULT_RULES,		"rules" },
+  { TESTCASE_RESULT_GENID,		"genid" },
+  { TESTCASE_RESULT_REASON,		"reason" },
   { 0, 0 }
 };
 
@@ -106,6 +113,8 @@ static struct solverflags2str {
   { SOLVER_FLAG_BREAK_ORPHANS,              "breakorphans", 0 },
   { SOLVER_FLAG_FOCUS_INSTALLED,            "focusinstalled", 0 },
   { SOLVER_FLAG_YUM_OBSOLETES,              "yumobsoletes", 0 },
+  { SOLVER_FLAG_NEED_UPDATEPROVIDE,         "needupdateprovide", 0 },
+  { SOLVER_FLAG_URPM_REORDER,               "urpmreorder", 0 },
   { 0, 0, 0 }
 };
 
@@ -124,6 +133,7 @@ static struct poolflags2str {
   { POOL_FLAG_HAVEDISTEPOCH,                "havedistepoch", 0 },
   { POOL_FLAG_NOOBSOLETESMULTIVERSION,      "noobsoletesmultiversion", 0 },
   { POOL_FLAG_ADDFILEPROVIDESFILTERED,      "addfileprovidesfiltered", 0 },
+  { POOL_FLAG_NOWHATPROVIDESAUX,            "nowhatprovidesaux", 0 },
   { 0, 0, 0 }
 };
 
@@ -157,6 +167,15 @@ static struct selflags2str {
   { 0, 0 }
 };
 
+static const char *features[] = {
+#ifdef ENABLE_LINKED_PKGS
+  "linked_packages",
+#endif
+#ifdef ENABLE_COMPLEX_DEPS
+  "complex_deps",
+#endif
+  0
+};
 
 typedef struct strqueue {
   char **str;
@@ -284,151 +303,340 @@ strqueue_diff(Strqueue *sq1, Strqueue *sq2, Strqueue *osq)
     strqueue_pushjoin(osq, "+", sq2->str[j++], 0);
 }
 
-static inline int
-pool_isknownarch(Pool *pool, Id id)
+
+static const char *
+testcase_id2str(Pool *pool, Id id, int isname)
 {
-  if (!id || id == ID_EMPTY)
-    return 0;
-  if (id == ARCH_SRC || id == ARCH_NOSRC || id == ARCH_NOARCH)
-    return 1;
-  if (!pool->id2arch || (id > pool->lastarch || !pool->id2arch[id]))
-    return 0;
-  return 1;
+  const char *s = pool_id2str(pool, id);
+  const char *ss;
+  char *s2, *s2p;
+  int bad = 0, paren = 0, parenbad = 0;
+
+  if (id == 0)
+    return "<NULL>";
+  if (id == 1)
+    return "\\00";
+  if (strchr("[(<=>!", *s))
+    bad++;
+  if (!strncmp(s, "namespace:", 10))
+    bad++;
+  for (ss = s + bad; *ss; ss++)
+    {
+      if (*ss == ' ' || *ss == '\\' || *(unsigned char *)ss < 32 || *ss == '(' || *ss == ')')
+        bad++;
+      if (*ss == '(')
+	paren = paren == 0 ? 1 : -1;
+      else if (*ss == ')')
+	{
+	  paren = paren == 1 ? 0 : -1;
+	  if (!paren)
+	    parenbad += 2;
+	}
+    }
+  if (isname && ss - s > 4 && !strcmp(ss - 4, ":any"))
+    bad++;
+  if (!paren && !(bad - parenbad))
+    return s;
+
+  /* we need escaping! */
+  s2 = s2p = pool_alloctmpspace(pool, strlen(s) + bad * 2 + 1);
+  if (!strncmp(s, "namespace:", 10))
+    {
+      strcpy(s2p, "namespace\\3a");
+      s2p += 12;
+      s += 10;
+    }
+  ss = s;
+  for (; *ss; ss++)
+    {
+      *s2p++ = *ss;
+      if ((ss == s && strchr("[(<=>!", *s)) || *ss == ' ' || *ss == '\\' || *(unsigned char *)ss < 32 || *ss == '(' || *ss == ')')
+	{
+	  s2p[-1] = '\\';
+	  solv_bin2hex((unsigned char *)ss, 1, s2p);
+	  s2p += 2;
+	}
+    }
+  *s2p = 0;
+  if (isname && s2p - s2 > 4 && !strcmp(s2p - 4, ":any"))
+    strcpy(s2p - 4, "\\3aany");
+  return s2;
 }
 
-static Id
-testcase_str2dep_simple(Pool *pool, const char **sp)
-{
-  const char *s = *sp;
-  const char *n, *a;
-  Id id, evr;
-  int flags;
+struct oplist {
+  Id flags;
+  const char *opname;
+} oplist[] = {
+  { REL_EQ, "=" },
+  { REL_GT | REL_LT | REL_EQ, "<=>" },
+  { REL_LT | REL_EQ, "<=" },
+  { REL_GT | REL_EQ, ">=" },
+  { REL_GT, ">" },
+  { REL_GT | REL_LT, "<>" },
+  { REL_AND,   "&" },
+  { REL_OR ,   "|" },
+  { REL_WITH , "+" },
+  { REL_NAMESPACE , "<NAMESPACE>" },
+  { REL_ARCH,       "." },
+  { REL_MULTIARCH,  "<MULTIARCH>" },
+  { REL_FILECONFLICT,  "<FILECONFLICT>" },
+  { REL_COND,  "<IF>" },
+  { REL_COMPAT,  "compat >=" },
+  { REL_KIND,  "<KIND>" },
+  { REL_ELSE, "<ELSE>" },
+  { REL_LT, "<" },
+  { 0, 0 }
+};
 
-  while (*s == ' ' || *s == '\t')
-    s++;
-  n = s;
-  while (*s && *s != ' ' && *s != '\t' && *s != '<' && *s != '=' && *s != '>')
+static const char *
+testcase_dep2str_complex(Pool *pool, Id id, int addparens)
+{
+  Reldep *rd;
+  char *s;
+  const char *s2;
+  int needparens;
+  struct oplist *op;
+
+  if (!ISRELDEP(id))
+    return testcase_id2str(pool, id, 1);
+  rd = GETRELDEP(pool, id);
+
+  /* check for special shortcuts */
+  if (rd->flags == REL_NAMESPACE && !ISRELDEP(rd->name) && !strncmp(pool_id2str(pool, rd->name), "namespace:", 10))
     {
-      if (*s == '(')
-	{
-	  while (*s && *s != ')')
-	    s++;
-	  continue;
-	}
-      s++;
+      const char *ns = pool_id2str(pool, rd->name);
+      int nslen = strlen(ns);
+      /* special namespace formatting */
+      const char *evrs = testcase_dep2str_complex(pool, rd->evr, 0);
+      s = pool_tmpappend(pool, evrs, ns, "()");
+      memmove(s + nslen + 1, s, strlen(s) - nslen - 2);
+      memcpy(s, ns, nslen);
+      s[nslen] = '(';
+      return s;
     }
-  if ((a = strchr(n, '.')) != 0 && a + 1 < s && s[-1] != ')')
+  if (rd->flags == REL_MULTIARCH && !ISRELDEP(rd->name) && rd->evr == ARCH_ANY)
     {
-      Id archid = pool_strn2id(pool, a + 1, s - (a + 1), 0);
-      if (pool_isknownarch(pool, archid))
-	{
-          id = pool_strn2id(pool, n, a - n, 1);
-	  id = pool_rel2id(pool, id, archid, REL_ARCH, 1);
-	}
-      else
-        id = pool_strn2id(pool, n, s - n, 1);
+      /* special :any suffix */
+      const char *ns = testcase_id2str(pool, rd->name, 1);
+      return pool_tmpappend(pool, ns, ":any", 0);
     }
-  else if (s - n > 4 && s[-4] == ':' && !strncmp(s - 4, ":any", 4))
+
+  needparens = 0;
+  if (ISRELDEP(rd->name))
     {
-      id = pool_strn2id(pool, n, s - n - 4, 1);
-      id = pool_rel2id(pool, id, ARCH_ANY, REL_MULTIARCH, 1);
+      Reldep *rd2 = GETRELDEP(pool, rd->name);
+      needparens = 1;
+      if (rd->flags > 7 && rd->flags != REL_COMPAT && rd2->flags && rd2->flags <= 7)
+	needparens = 0;
+    }
+  s = (char *)testcase_dep2str_complex(pool, rd->name, needparens);
+
+  if (addparens)
+    {
+      s = pool_tmpappend(pool, s, "(", 0);
+      memmove(s + 1, s, strlen(s + 1));
+      s[0] = '(';
+    }
+  for (op = oplist; op->flags; op++)
+    if (rd->flags == op->flags)
+      break;
+  if (op->flags)
+    {
+      s = pool_tmpappend(pool, s, " ", op->opname);
+      s = pool_tmpappend(pool, s, " ", 0);
     }
   else
-    id = pool_strn2id(pool, n, s - n, 1);
-  if (!*s)
     {
-      *sp = s;
-      return id;
+      char buf[64];
+      sprintf(buf, " <%u> ", rd->flags);
+      s = pool_tmpappend(pool, s, buf, 0);
     }
-  while (*s == ' ' || *s == '\t')
-    s++;
-  flags = 0;
-  if (*s == '!' && s[1] == '=')	/* support != as synonym for <> */
+
+  needparens = 0;
+  if (ISRELDEP(rd->evr))
     {
-      flags = REL_LT | REL_GT;
-      s += 2;
+      Reldep *rd2 = GETRELDEP(pool, rd->evr);
+      needparens = 1;
+      if (rd->flags > 7 && rd2->flags && rd2->flags <= 7)
+	needparens = 0;
+      if (rd->flags == REL_AND && rd2->flags == REL_AND)
+	needparens = 0;	/* chain */
+      if (rd->flags == REL_OR && rd2->flags == REL_OR)
+	needparens = 0;	/* chain */
+      if (rd->flags > 0 && rd->flags < 8 && rd2->flags == REL_COMPAT)
+	needparens = 0;	/* chain */
     }
-  for (;;s++)
+  if (!ISRELDEP(rd->evr))
+    s2 = testcase_id2str(pool, rd->evr, 0);
+  else
+    s2 = testcase_dep2str_complex(pool, rd->evr, needparens);
+  if (addparens)
+    s = pool_tmpappend(pool, s, s2, ")");
+  else
+    s = pool_tmpappend(pool, s, s2, 0);
+  pool_freetmpspace(pool, s2);
+  return s;
+}
+
+const char *
+testcase_dep2str(Pool *pool, Id id)
+{
+  return testcase_dep2str_complex(pool, id, 0);
+}
+
+
+/* Convert a simple string. Also handle the :any suffix */
+static Id
+testcase_str2dep_simple(Pool *pool, const char **sp, int isname)
+{
+  int haveesc = 0;
+  int paren = 0;
+  int isany = 0;
+  Id id;
+  const char *s;
+  for (s = *sp; *s; s++)
     {
-      if (*s == '<')
-	flags |= REL_LT;
-      else if (*s == '=')
-	flags |= REL_EQ;
-      else if (*s == '>')
-	flags |= REL_GT;
-      else
+      if (*s == '\\')
+	haveesc++;
+      if (*s == ' ' || *(unsigned char *)s < 32)
+	break;
+      if (*s == '(')
+	paren++;
+      if (*s == ')' && paren-- <= 0)
 	break;
     }
-  if (!flags)
+  if (isname && s - *sp > 4 && !strncmp(s - 4, ":any", 4))
     {
-      *sp = s;
-      return id;
+      isany = 1;
+      s -= 4;
     }
-  while (*s == ' ' || *s == '\t')
-    s++;
-  n = s;
-  while (*s && *s != ' ' && *s != '\t')
-    s++;
-  evr = pool_strn2id(pool, n, s - n, 1);
-  if (*s == ' ' && !strcmp(s, " compat >= "))
+  if (!haveesc)
     {
-      s += 11;
-      while (*s == ' ' || *s == '\t')
-	s++;
-      n = s;
-      while (*s && *s != ' ' && *s != '\t')
-	s++;
-      evr = pool_rel2id(pool, evr, pool_strn2id(pool, n, s - n, 1), REL_COMPAT, 1);
-    }
-  *sp = s;
-  return pool_rel2id(pool, id, evr, flags, 1);
-}
-
-static Id
-testcase_str2dep_complex(Pool *pool, const char **sp)
-{
-  const char *s = *sp;
-  Id id;
-#ifdef ENABLE_COMPLEX_DEPS
-  while (*s == ' ' || *s == '\t')
-    s++;
-  if (*s == '(')
-    {
-      s++;
-      id = testcase_str2dep_complex(pool, &s);
-      if (*s == ')')
-	s++;
-      while (*s == ' ' || *s == '\t')
-	s++;
+      if (s - *sp == 6 && !strncmp(*sp, "<NULL>", 6))
+	id = 0;
+      else
+        id = pool_strn2id(pool, *sp, s - *sp, 1);
     }
+  else if (s - *sp == 3 && !strncmp(*sp, "\\00", 3))
+    id = 1;
   else
-#endif
-    id = testcase_str2dep_simple(pool, &s);
-  if (*s == '|')
     {
-      s++;
-      id = pool_rel2id(pool, id, testcase_str2dep_complex(pool, &s), REL_OR, 1);
+      char buf[128], *bp, *bp2;
+      const char *sp2;
+      bp = s - *sp >= 128 ? solv_malloc(s - *sp + 1) : buf;
+      for (bp2 = bp, sp2 = *sp; sp2 < s;)
+	{
+	  *bp2++ = *sp2++;
+	  if (bp2[-1] == '\\')
+	    solv_hex2bin(&sp2, (unsigned char *)bp2 - 1, 1);
+	}
+      *bp2 = 0;
+      id = pool_str2id(pool, bp, 1);
+      if (bp != buf)
+	solv_free(bp);
     }
-  else if (*s == '&')
+  if (isany)
     {
-      s++;
-      id = pool_rel2id(pool, id, testcase_str2dep_complex(pool, &s), REL_AND, 1);
-    }
-  else if (*s == 'I' && s[1] == 'F' && (s[2] == ' ' || s[2] == '\t'))
-    {
-      s += 2;
-      id = pool_rel2id(pool, id, testcase_str2dep_complex(pool, &s), REL_COND, 1);
+      id = pool_rel2id(pool, id, ARCH_ANY, REL_MULTIARCH, 1);
+      s += 4;
     }
   *sp = s;
   return id;
 }
 
+
+static Id
+testcase_str2dep_complex(Pool *pool, const char **sp, int relop)
+{
+  const char *s = *sp;
+  Id flags, id, id2, namespaceid = 0;
+  struct oplist *op;
+
+  while (*s == ' ' || *s == '\t')
+    s++;
+  if (!strncmp(s, "namespace:", 10))
+    {
+      /* special namespace hack */
+      const char *s2;
+      for (s2 = s + 10; *s2 && *s2 != '('; s2++)
+	;
+      if (*s2 == '(')
+	{
+	  namespaceid = pool_strn2id(pool, s, s2 - s, 1);
+	  s = s2;
+	}
+    }
+  if (*s == '(')
+    {
+      s++;
+      id = testcase_str2dep_complex(pool, &s, 0);
+      if (!s || *s != ')')
+	{
+	  *sp = 0;
+	  return 0;
+	}
+      s++;
+    }
+  else
+    id = testcase_str2dep_simple(pool, &s, relop ? 0 : 1);
+  if (namespaceid)
+    id = pool_rel2id(pool, namespaceid, id, REL_NAMESPACE, 1);
+    
+  for (;;)
+    {
+      while (*s == ' ' || *s == '\t')
+	s++;
+      if (!*s || *s == ')' || (relop && strncmp(s, "compat >= ", 10) != 0))
+	{
+	  *sp = s;
+	  return id;
+	}
+
+      /* we have an op! Find the end */
+      flags = -1;
+      if (s[0] == '<' && (s[1] >= '0' && s[1] <= '9'))
+	{
+	  const char *s2;
+	  for (s2 = s + 1; *s2 >= '0' && *s2 <= '9'; s2++)
+	    ;
+	  if (*s2 == '>')
+	    {
+	      flags = strtoul(s + 1, 0, 10);
+	      s = s2 + 1;
+	    }
+	}
+      if (flags == -1)
+	{
+	  for (op = oplist; op->flags; op++)
+	    if (!strncmp(s, op->opname, strlen(op->opname)))
+	      break;
+	  if (!op->flags)
+	    {
+	      *sp = 0;
+	      return 0;
+	    }
+	  flags = op->flags;
+	  s += strlen(op->opname);
+	}
+      id2 = testcase_str2dep_complex(pool, &s, flags > 0 && flags < 8);
+      if (!s)
+	{
+	  *sp = 0;
+	  return 0;
+	}
+      id = pool_rel2id(pool, id, id2, flags, 1);
+    }
+}
+
 Id
 testcase_str2dep(Pool *pool, const char *s)
 {
-  return testcase_str2dep_complex(pool, &s);
+  Id id = testcase_str2dep_complex(pool, &s, 0);
+  return s && !*s ? id : 0;
 }
 
+/**********************************************************/
+
 const char *
 testcase_repoid2str(Pool *pool, Id repoid)
 {
@@ -597,12 +805,12 @@ testcase_job2str(Pool *pool, Id how, Id what)
   else if (select == SOLVER_SOLVABLE_NAME)
     {
       selstr = " name ";
-      pkgstr = pool_dep2str(pool, what);
+      pkgstr = testcase_dep2str(pool, what);
     }
   else if (select == SOLVER_SOLVABLE_PROVIDES)
     {
       selstr = " provides ";
-      pkgstr = pool_dep2str(pool, what);
+      pkgstr = testcase_dep2str(pool, what);
     }
   else if (select == SOLVER_SOLVABLE_ONE_OF)
     {
@@ -920,14 +1128,13 @@ static void
 writedeps(Repo *repo, FILE *fp, const char *tag, Id key, Solvable *s, Offset off)
 {
   Pool *pool = repo->pool;
-  Id id, *dp, *prvdp;
+  Id id, *dp;
   int tagwritten = 0;
   const char *idstr;
 
   if (!off)
     return;
   dp = repo->idarraydata + off;
-  prvdp = 0;
   while ((id = *dp++) != 0)
     {
       if (key == SOLVABLE_REQUIRES && id == SOLVABLE_PREREQMARKER)
@@ -939,68 +1146,8 @@ writedeps(Repo *repo, FILE *fp, const char *tag, Id key, Solvable *s, Offset off
 	  continue;
 	}
       if (key == SOLVABLE_PROVIDES && id == SOLVABLE_FILEMARKER)
-	{
-	  prvdp = dp;
-	  continue;
-	}
-      idstr = pool_dep2str(pool, id);
-      if (ISRELDEP(id))
-	{
-	  Reldep *rd = GETRELDEP(pool, id);
-	  if (key == SOLVABLE_CONFLICTS && rd->flags == REL_NAMESPACE && rd->name == NAMESPACE_OTHERPROVIDERS)
-	    {
-	      if (!strncmp(idstr, "namespace:", 10))
-		idstr += 10;
-	    }
-	  if (key == SOLVABLE_SUPPLEMENTS)
-	    {
-	      if (rd->flags == REL_NAMESPACE && rd->name == NAMESPACE_FILESYSTEM)
-		{
-		  if (!strncmp(idstr, "namespace:", 10))
-		    idstr += 10;
-		}
-	      else if (rd->flags == REL_NAMESPACE && rd->name == NAMESPACE_MODALIAS)
-		{
-		  if (!strncmp(idstr, "namespace:", 10))
-		    idstr += 10;
-		}
-	      else if (rd->flags == REL_AND)
-		{
-		  /* either packageand chain or modalias */
-		  idstr = 0;
-		  if (ISRELDEP(rd->evr))
-		    {
-		      Reldep *mrd = GETRELDEP(pool, rd->evr);
-		      if (mrd->flags == REL_NAMESPACE && mrd->name == NAMESPACE_MODALIAS)
-			{
-			  idstr = pool_tmpjoin(pool, "modalias(", pool_dep2str(pool, rd->name), ":");
-			  idstr = pool_tmpappend(pool, idstr, pool_dep2str(pool, mrd->evr), ")");
-			}
-		      else if (mrd->flags >= 8)
-			continue;
-		    }
-		  if (!idstr)
-		    {
-		      /* must be and chain */
-		      idstr = pool_dep2str(pool, rd->evr);
-		      for (;;)
-			{
-			  id = rd->name;
-			  if (!ISRELDEP(id))
-			    break;
-			  rd = GETRELDEP(pool, id);
-			  if (rd->flags != REL_AND)
-			    break;
-			  idstr = pool_tmpjoin(pool, pool_dep2str(pool, rd->evr), ":", idstr);
-			}
-		      idstr = pool_tmpjoin(pool, pool_dep2str(pool, id), ":", idstr);
-		      idstr = pool_tmpjoin(pool, "packageand(", idstr, ")");
-		    }
-		}
-	      else if (rd->flags >= 8)
-		continue;
-	    }
-	}
+	continue;
+      idstr = testcase_dep2str(pool, id);
       if (!tagwritten)
 	{
 	  fprintf(fp, "+%s\n", tag);
@@ -1008,36 +1155,31 @@ writedeps(Repo *repo, FILE *fp, const char *tag, Id key, Solvable *s, Offset off
 	}
       fprintf(fp, "%s\n", idstr);
     }
-  if (key == SOLVABLE_PROVIDES)
+  if (tagwritten)
+    fprintf(fp, "-%s\n", tag);
+}
+
+static void
+writefilelist(Repo *repo, FILE *fp, const char *tag, Solvable *s)
+{
+  Pool *pool = repo->pool;
+  int tagwritten = 0;
+  Dataiterator di;
+
+  dataiterator_init(&di, pool, repo, s - pool->solvables, SOLVABLE_FILELIST, 0, 0);
+  while (dataiterator_step(&di))
     {
-      /* add the filelist */
-      Dataiterator di;
-      dataiterator_init(&di, pool, repo, s - pool->solvables, SOLVABLE_FILELIST, 0, 0);
-      while (dataiterator_step(&di))
+      const char *s = repodata_dir2str(di.data, di.kv.id, di.kv.str);
+      if (!tagwritten)
 	{
-	  const char *s = repodata_dir2str(di.data, di.kv.id, di.kv.str);
-	  if (prvdp)
-	    {
-	      Id id = pool_str2id(pool, s, 0);
-	      if (id)
-		{
-		  for (dp = prvdp; *dp; dp++)
-		    if (*dp == id)
-		      break;
-		  if (*dp)
-		    continue;	/* already included */
-		}
-	    }
-	  if (!tagwritten)
-	    {
-	      fprintf(fp, "+%s", tag);
-	      tagwritten = 1;
-	    }
-	  fprintf(fp, "%s\n", s);
+	  fprintf(fp, "+%s\n", tag);
+	  tagwritten = 1;
 	}
+      fprintf(fp, "%s\n", s);
     }
   if (tagwritten)
     fprintf(fp, "-%s\n", tag);
+  dataiterator_free(&di);
 }
 
 int
@@ -1052,8 +1194,10 @@ testcase_write_testtags(Repo *repo, FILE *fp)
   const char *release;
   const char *tmp;
   unsigned int ti;
+  Queue q;
 
-  fprintf(fp, "=Ver: 2.0\n");
+  fprintf(fp, "=Ver: 3.0\n");
+  queue_init(&q);
   FOR_REPO_SOLVABLES(repo, p, s)
     {
       name = pool_id2str(pool, s->name);
@@ -1074,12 +1218,22 @@ testcase_write_testtags(Repo *repo, FILE *fp)
       writedeps(repo, fp, "Sup:", SOLVABLE_SUPPLEMENTS, s, s->supplements);
       writedeps(repo, fp, "Sug:", SOLVABLE_SUGGESTS, s, s->suggests);
       writedeps(repo, fp, "Enh:", SOLVABLE_ENHANCES, s, s->enhances);
+      if (solvable_lookup_idarray(s, SOLVABLE_PREREQ_IGNOREINST, &q))
+	{
+	  int i;
+	  fprintf(fp, "+Ipr:\n");
+	  for (i = 0; i < q.count; i++)
+	    fprintf(fp, "%s\n", testcase_dep2str(pool, q.elements[i]));
+	  fprintf(fp, "-Ipr:\n");
+	}
       if (s->vendor)
 	fprintf(fp, "=Vnd: %s\n", pool_id2str(pool, s->vendor));
       ti = solvable_lookup_num(s, SOLVABLE_BUILDTIME, 0);
       if (ti)
 	fprintf(fp, "=Tim: %u\n", ti);
+      writefilelist(repo, fp, "Fls:", s);
     }
+  queue_free(&q);
   return 0;
 }
 
@@ -1091,7 +1245,7 @@ adddep(Repo *repo, Offset olddeps, char *str, Id marker)
 }
 
 static void
-finish_solvable(Pool *pool, Repodata *data, Solvable *s, char *filelist, int nfilelist)
+finish_v2_solvable(Pool *pool, Repodata *data, Solvable *s, char *filelist, int nfilelist)
 {
   if (nfilelist)
     {
@@ -1110,8 +1264,6 @@ finish_solvable(Pool *pool, Repodata *data, Solvable *s, char *filelist, int nfi
 	  repodata_add_dirstr(data, s - pool->solvables, SOLVABLE_FILELIST, did, p);
 	}
     }
-  if (s->name && s->arch != ARCH_SRC && s->arch != ARCH_NOSRC)
-    s->provides = repo_addid_dep(s->repo, s->provides, pool_rel2id(pool, s->name, s->evr, REL_EQ, 1), 0);
   s->supplements = repo_fix_supplements(s->repo, s->provides, s->supplements, 0);
   s->conflicts = repo_fix_conflicts(s->repo, s->conflicts);
 }
@@ -1132,6 +1284,8 @@ testcase_add_testtags(Repo *repo, FILE *fp, int flags)
   char *filelist = 0;
   int afilelist = 0;
   int nfilelist = 0;
+  int tagsversion = 0;
+  int addselfprovides = 1;	/* for compat reasons */
 
   data = repo_add_repodata(repo, flags);
   s = 0;
@@ -1181,9 +1335,18 @@ testcase_add_testtags(Repo *repo, FILE *fp, int flags)
       tag = line[1] << 16 | line[2] << 8 | line[3];
       switch(tag)
         {
+	case 'V' << 16 | 'e' << 8 | 'r':
+	  tagsversion = atoi(line + 6);
+	  addselfprovides = tagsversion < 3 || strstr(line + 6, "addselfprovides") != 0;
+	  break;
 	case 'P' << 16 | 'k' << 8 | 'g':
 	  if (s)
-	    finish_solvable(pool, data, s, filelist, nfilelist);
+	    {
+	      if (tagsversion == 2)
+		finish_v2_solvable(pool, data, s, filelist, nfilelist);
+	      if (addselfprovides && s->name && s->arch != ARCH_SRC && s->arch != ARCH_NOSRC)
+		s->provides = repo_addid_dep(s->repo, s->provides, pool_rel2id(pool, s->name, s->evr, REL_EQ, 1), 0);
+	    }
 	  nfilelist = 0;
 	  if (split(line + 5, sp, 5) != 4)
 	    break;
@@ -1213,27 +1376,44 @@ testcase_add_testtags(Repo *repo, FILE *fp, int flags)
 	  s->requires = adddep(repo, s->requires, line + 6, SOLVABLE_PREREQMARKER);
 	  break;
 	case 'P' << 16 | 'r' << 8 | 'v':
-	  if (line[6] == '/')
+	  /* version 2 had the file list at the end of the provides */
+	  if (tagsversion == 2)
 	    {
-	      int l = strlen(line + 6) + 1;
-	      if (nfilelist + l > afilelist)
+	      if (line[6] == '/')
 		{
-		  afilelist = nfilelist + l + 512;
-		  filelist = solv_realloc(filelist, afilelist);
+		  int l = strlen(line + 6) + 1;
+		  if (nfilelist + l > afilelist)
+		    {
+		      afilelist = nfilelist + l + 512;
+		      filelist = solv_realloc(filelist, afilelist);
+		    }
+		  memcpy(filelist + nfilelist, line + 6, l);
+		  nfilelist += l;
+		  break;
+		}
+	      if (nfilelist)
+		{
+		  int l;
+		  for (l = 0; l < nfilelist; l += strlen(filelist + l) + 1)
+		    s->provides = repo_addid_dep(repo, s->provides, pool_str2id(pool, filelist + l, 1), 0);
+		  nfilelist = 0;
 		}
-	      memcpy(filelist + nfilelist, line + 6, l);
-	      nfilelist += l;
-	      break;
-	    }
-	  if (nfilelist)
-	    {
-	      int l;
-	      for (l = 0; l < nfilelist; l += strlen(filelist + l) + 1)
-                s->provides = repo_addid_dep(repo, s->provides, pool_str2id(pool, filelist + l, 1), 0);
-              nfilelist = 0;
 	    }
 	  s->provides = adddep(repo, s->provides, line + 6, 0);
 	  break;
+	case 'F' << 16 | 'l' << 8 | 's':
+	  {
+	    char *p = strrchr(line + 6, '/');
+	    Id did;
+	    if (!p)
+	      break;
+	    *p++ = 0;
+	    did = repodata_str2dir(data, line + 6, 1);
+	    if (!did)
+	      did = repodata_str2dir(data, "/", 1);
+	    repodata_add_dirstr(data, s - pool->solvables, SOLVABLE_FILELIST, did, p);
+	    break;
+	  }
 	case 'O' << 16 | 'b' << 8 | 's':
 	  s->obsoletes = adddep(repo, s->obsoletes, line + 6, 0);
 	  break;
@@ -1252,12 +1432,23 @@ testcase_add_testtags(Repo *repo, FILE *fp, int flags)
 	case 'E' << 16 | 'n' << 8 | 'h':
 	  s->enhances = adddep(repo, s->enhances, line + 6, 0);
 	  break;
+	case 'I' << 16 | 'p' << 8 | 'r':
+	  {
+	    Id id = line[6] == '/' ? pool_str2id(pool, line + 6, 1) : testcase_str2dep(pool, line + 6);
+	    repodata_add_idarray(data, s - pool->solvables, SOLVABLE_PREREQ_IGNOREINST, id);
+	    break;
+	  }
         default:
 	  break;
         }
     }
   if (s)
-    finish_solvable(pool, data, s, filelist, nfilelist);
+    {
+      if (tagsversion == 2)
+	finish_v2_solvable(pool, data, s, filelist, nfilelist);
+      if (addselfprovides && s->name && s->arch != ARCH_SRC && s->arch != ARCH_NOSRC)
+        s->provides = repo_addid_dep(s->repo, s->provides, pool_rel2id(pool, s->name, s->evr, REL_EQ, 1), 0);
+    }
   solv_free(line);
   solv_free(filelist);
   repodata_free_dircache(data);
@@ -1366,7 +1557,11 @@ testcase_setsolverflags(Solver *solv, const char *str)
 	  pool_debug(solv->pool, SOLV_ERROR, "setsolverflags: unknown flag '%.*s'\n", (int)(p - s), s);
 	  return 0;
 	}
-      solver_set_flag(solv, solverflags2str[i].flag, v);
+      if (solver_set_flag(solv, solverflags2str[i].flag, v) == -1)
+	{
+	  pool_debug(solv->pool, SOLV_ERROR, "setsolverflags: unsupported flag '%s'\n", solverflags2str[i].str);
+	  return 0;
+	}
     }
   return 1;
 }
@@ -1470,6 +1665,33 @@ testcase_solutionid(Solver *solv, Id problem, Id solution)
   return s;
 }
 
+static const char *
+testcase_alternativeid(Solver *solv, int type, Id id, Id from)
+{
+  const char *s;
+  Pool *pool = solv->pool;
+  Chksum *chk;
+  const unsigned char *md5;
+  int md5l;
+  chk = solv_chksum_create(REPOKEY_TYPE_MD5);
+  if (type == SOLVER_ALTERNATIVE_TYPE_RECOMMENDS)
+    {
+      s = testcase_solvid2str(pool, from);
+      solv_chksum_add(chk, s, strlen(s) + 1);
+      s = testcase_dep2str(pool, id);
+      solv_chksum_add(chk, s, strlen(s) + 1);
+    }
+  else if (type == SOLVER_ALTERNATIVE_TYPE_RULE)
+    {
+      s = testcase_ruleid(solv, id);
+      solv_chksum_add(chk, s, strlen(s) + 1);
+    }
+  md5 = solv_chksum_get(chk, &md5l);
+  s = pool_bin2hex(pool, md5, 4);
+  chk = solv_chksum_free(chk, 0);
+  return s;
+}
+
 static struct class2str {
   Id class;
   const char *str;
@@ -1486,6 +1708,62 @@ static struct class2str {
   { 0, 0 }
 };
 
+static struct reason2str {
+  Id reason;
+  const char *str;
+} reason2str[] = {
+  { SOLVER_REASON_UNRELATED,		"unrelated" },
+  { SOLVER_REASON_UNIT_RULE,		"unit" },
+  { SOLVER_REASON_KEEP_INSTALLED,	"keep" },
+  { SOLVER_REASON_RESOLVE_JOB,		"job" },
+  { SOLVER_REASON_UPDATE_INSTALLED,	"update" },
+  { SOLVER_REASON_CLEANDEPS_ERASE,	"cleandeps" },
+  { SOLVER_REASON_RESOLVE,		"resolve" },
+  { SOLVER_REASON_WEAKDEP,		"weakdep" },
+  { SOLVER_REASON_RESOLVE_ORPHAN,	"orphan" },
+
+  { SOLVER_REASON_RECOMMENDED,		"recommended" },
+  { SOLVER_REASON_SUPPLEMENTED,		"supplemented" },
+  { 0, 0 }
+};
+
+static const char *
+testcase_reason2str(Id reason)
+{
+  int i;
+  for (i = 0; reason2str[i].str; i++)
+    if (reason == reason2str[i].reason)
+      return reason2str[i].str;
+  return "?";
+}
+
+static int
+dump_genid(Pool *pool, Strqueue *sq, Id id, int cnt)
+{
+  struct oplist *op;
+  char cntbuf[20];
+  const char *s;
+
+  if (ISRELDEP(id))
+    {
+      Reldep *rd = GETRELDEP(pool, id);
+      for (op = oplist; op->flags; op++)
+	if (rd->flags == op->flags)
+	  break;
+      cnt = dump_genid(pool, sq, rd->name, cnt);
+      cnt = dump_genid(pool, sq, rd->evr, cnt);
+      sprintf(cntbuf, "genid %2d: genid ", cnt++);
+      s = pool_tmpjoin(pool, cntbuf, "op ", op->flags ? op->opname : "unknown");
+    }
+  else
+    {
+      sprintf(cntbuf, "genid %2d: genid ", cnt++);
+      s = pool_tmpjoin(pool, cntbuf, id ? "lit " : "null", id ? pool_id2str(pool, id) : 0);
+    }
+  strqueue_push(sq, s);
+  return cnt;
+}
+
 char *
 testcase_solverresult(Solver *solv, int resultflags)
 {
@@ -1626,7 +1904,194 @@ testcase_solverresult(Solver *solv, int resultflags)
       queue_free(&q);
       queue_free(&qf);
     }
+  if ((resultflags & TESTCASE_RESULT_ALTERNATIVES) != 0)
+    {
+      char *altprefix;
+      Queue q, rq;
+      int cnt;
+      Id alternative;
+      queue_init(&q);
+      queue_init(&rq);
+      cnt = solver_alternatives_count(solv);
+      for (alternative = 1; alternative <= cnt; alternative++)
+	{
+	  Id id, from, chosen;
+	  char num[20];
+	  int type = solver_get_alternative(solv, alternative, &id, &from, &chosen, &q, 0);
+	  altprefix = solv_dupjoin("alternative ", testcase_alternativeid(solv, type, id, from), " ");
+	  strcpy(num, " 0 ");
+	  if (type == SOLVER_ALTERNATIVE_TYPE_RECOMMENDS)
+	    {
+	      char *s = pool_tmpjoin(pool, altprefix, num, testcase_solvid2str(pool, from));
+	      s = pool_tmpappend(pool, s, " recommends ", testcase_dep2str(pool, id));
+	      strqueue_push(&sq, s);
+	    }
+	  else if (type == SOLVER_ALTERNATIVE_TYPE_RULE)
+	    {
+	      /* map choice rules back to pkg rules */
+	      if (solver_ruleclass(solv, id) == SOLVER_RULE_CHOICE)
+		id = solver_rule2pkgrule(solv, id);
+	      solver_allruleinfos(solv, id, &rq);
+	      for (i = 0; i < rq.count; i += 4)
+		{
+		  int rtype = rq.elements[i];
+		  if ((rtype & SOLVER_RULE_TYPEMASK) == SOLVER_RULE_JOB)
+		    {
+		      const char *js = testcase_job2str(pool, rq.elements[i + 2], rq.elements[i + 3]);
+		      char *s = pool_tmpjoin(pool, altprefix, num, " job ");
+		      s = pool_tmpappend(pool, s, js, 0);
+		      strqueue_push(&sq, s);
+		    }
+		  else if (rtype == SOLVER_RULE_PKG_REQUIRES)
+		    {
+		      char *s = pool_tmpjoin(pool, altprefix, num, testcase_solvid2str(pool, rq.elements[i + 1]));
+		      s = pool_tmpappend(pool, s, " requires ", testcase_dep2str(pool, rq.elements[i + 3]));
+		      strqueue_push(&sq, s);
+		    }
+		}
+	    }
+	  for (i = 0; i < q.count; i++)
+	    {
+	      Id p = q.elements[i];
+	      if (i >= 9)
+	        num[0] = '0' + (i + 1) / 10;
+	      num[1] = '0' + (i + 1) % 10;
+	      if (-p == chosen)
+		s = pool_tmpjoin(pool, altprefix, num, "+ ");
+	      else if (p < 0)
+	        s = pool_tmpjoin(pool, altprefix, num, "- ");
+	      else if (p >= 0)
+	        s = pool_tmpjoin(pool, altprefix, num, "  ");
+	      s = pool_tmpappend(pool, s,  testcase_solvid2str(pool, p < 0 ? -p : p), 0);
+	      strqueue_push(&sq, s);
+	    }
+	  solv_free(altprefix);
+	}
+      queue_free(&q);
+      queue_free(&rq);
+    }
+  if ((resultflags & TESTCASE_RESULT_RULES) != 0)
+    {
+      /* dump all rules */
+      Id rid;
+      SolverRuleinfo rclass;
+      Queue q;
+      int i;
 
+      queue_init(&q);
+      for (rid = 1; (rclass = solver_ruleclass(solv, rid)) != SOLVER_RULE_UNKNOWN; rid++)
+	{
+	  char *prefix;
+	  switch (rclass)
+	    {
+	    case SOLVER_RULE_PKG:
+	      prefix = "pkg ";
+	      break;
+	    case SOLVER_RULE_UPDATE:
+	      prefix = "update ";
+	      break;
+	    case SOLVER_RULE_FEATURE:
+	      prefix = "feature ";
+	      break;
+	    case SOLVER_RULE_JOB:
+	      prefix = "job ";
+	      break;
+	    case SOLVER_RULE_DISTUPGRADE:
+	      prefix = "distupgrade ";
+	      break;
+	    case SOLVER_RULE_INFARCH:
+	      prefix = "infarch ";
+	      break;
+	    case SOLVER_RULE_CHOICE:
+	      prefix = "choice ";
+	      break;
+	    case SOLVER_RULE_LEARNT:
+	      prefix = "learnt ";
+	      break;
+	    case SOLVER_RULE_BEST:
+	      prefix = "best ";
+	      break;
+	    case SOLVER_RULE_YUMOBS:
+	      prefix = "yumobs ";
+	      break;
+	    default:
+	      prefix = "unknown ";
+	      break;
+	    }
+	  prefix = solv_dupjoin("rule ", prefix, testcase_ruleid(solv, rid));
+	  solver_ruleliterals(solv, rid, &q);
+	  if (rclass == SOLVER_RULE_FEATURE && q.count == 1 && q.elements[0] == -SYSTEMSOLVABLE)
+	    continue;
+	  for (i = 0; i < q.count; i++)
+	    {
+	      Id p = q.elements[i];
+	      const char *s;
+	      if (p < 0)
+		s = pool_tmpjoin(pool, prefix, " -", testcase_solvid2str(pool, -p));
+	      else
+		s = pool_tmpjoin(pool, prefix, "  ", testcase_solvid2str(pool, p));
+	      strqueue_push(&sq, s);
+	    }
+	  solv_free(prefix);
+	}
+      queue_free(&q);
+    }
+  if ((resultflags & TESTCASE_RESULT_GENID) != 0)
+    {
+      for (i = 0 ; i < solv->job.count; i += 2)
+	{
+	  Id id, id2;
+	  if (solv->job.elements[i] != (SOLVER_NOOP | SOLVER_SOLVABLE_PROVIDES))
+	    continue;
+	  id = solv->job.elements[i + 1];
+	  s = testcase_dep2str(pool, id);
+	  strqueue_push(&sq, pool_tmpjoin(pool, "genid dep ", s, 0));
+	  if ((id2 = testcase_str2dep(pool, s)) != id)
+	    {
+	      s = pool_tmpjoin(pool, "genid roundtrip error: ", testcase_dep2str(pool, id2), 0);
+	      strqueue_push(&sq, s);
+	    }
+	  dump_genid(pool, &sq, id, 1);
+	}
+    }
+  if ((resultflags & TESTCASE_RESULT_REASON) != 0)
+    {
+      Queue whyq;
+      queue_init(&whyq);
+      FOR_POOL_SOLVABLES(p)
+	{
+	  Id info, p2, id;
+          int reason = solver_describe_decision(solv, p, &info);
+	  if (reason == SOLVER_REASON_UNRELATED)
+	    continue;
+	  if (reason == SOLVER_REASON_WEAKDEP)
+	    {
+	      solver_describe_weakdep_decision(solv, p, &whyq);
+	      if (whyq.count)
+		{
+		  for (i = 0; i < whyq.count; i += 3)
+		    {
+		      reason = whyq.elements[i];
+		      p2 = whyq.elements[i + 1];
+		      id = whyq.elements[i + 2];
+		      s = pool_tmpjoin(pool, "reason ", testcase_solvid2str(pool, p), 0);
+		      s = pool_tmpappend(pool, s, " ", testcase_reason2str(reason));
+		      s = pool_tmpappend(pool, s, " ", testcase_dep2str(pool, id));
+		      if (p2)
+		        s = pool_tmpappend(pool, s, " ", testcase_solvid2str(pool, p2));
+		      strqueue_push(&sq, s);
+		    }
+		  continue;
+		}
+	    }
+	  s = pool_tmpjoin(pool, "reason ", testcase_solvid2str(pool, p), 0);
+	  s = pool_tmpappend(pool, s, " ", testcase_reason2str(reason));
+	  if (info)
+	    s = pool_tmpappend(pool, s, " ", testcase_ruleid(solv, info));
+	  strqueue_push(&sq, s);
+	}
+      queue_free(&whyq);
+    }
   strqueue_sort(&sq);
   result = strqueue_join(&sq);
   strqueue_free(&sq);
@@ -1635,7 +2100,7 @@ testcase_solverresult(Solver *solv, int resultflags)
 
 
 int
-testcase_write(Solver *solv, char *dir, int resultflags, const char *testcasename, const char *resultname)
+testcase_write(Solver *solv, const char *dir, int resultflags, const char *testcasename, const char *resultname)
 {
   Pool *pool = solv->pool;
   Repo *repo;
@@ -1961,7 +2426,7 @@ str2resultflags(Pool *pool, char *s)	/* modifies the string! */
 }
 
 Solver *
-testcase_read(Pool *pool, FILE *fp, char *testcase, Queue *job, char **resultp, int *resultflagsp)
+testcase_read(Pool *pool, FILE *fp, const char *testcase, Queue *job, char **resultp, int *resultflagsp)
 {
   Solver *solv;
   char *buf, *bufp;
@@ -1973,6 +2438,9 @@ testcase_read(Pool *pool, FILE *fp, char *testcase, Queue *job, char **resultp,
   int prepared = 0;
   int closefp = !fp;
   int poolflagsreset = 0;
+  int missing_features = 0;
+  Id *genid = 0;
+  int ngenid = 0;
 
   if (!fp && !(fp = fopen(testcase, "r")))
     {
@@ -2095,20 +2563,38 @@ testcase_read(Pool *pool, FILE *fp, char *testcase, Queue *job, char **resultp,
       else if (!strcmp(pieces[0], "system") && npieces >= 3)
 	{
 	  int i;
-	  prepared = 0;
+
 	  /* must set the disttype before the arch */
-	  for (i = 0; disttype2str[i].str != 0; i++)
-	    if (!strcmp(disttype2str[i].str, pieces[2]))
-	      break;
-	  if (!disttype2str[i].str)
-	    pool_debug(pool, SOLV_ERROR, "testcase_read: system: unknown disttype '%s'\n", pieces[2]);
-	  else if (pool->disttype != disttype2str[i].type)
+	  prepared = 0;
+	  if (strcmp(pieces[2], "*") != 0)
 	    {
+	      char *dp = pieces[2];
+	      while (dp && *dp)
+		{
+		  char *dpe = strchr(dp, ',');
+		  if (dpe)
+		    *dpe = 0;
+		  for (i = 0; disttype2str[i].str != 0; i++)
+		    if (!strcmp(disttype2str[i].str, dp))
+		      break;
+		  if (dpe)
+		    *dpe++ = ',';
+		  if (disttype2str[i].str)
+		    {
 #ifdef MULTI_SEMANTICS
-	      pool_setdisttype(pool, disttype2str[i].type);
-#else
-	      pool_debug(pool, SOLV_ERROR, "testcase_read: system: cannot change disttype to '%s'\n", pieces[2]);
+		      if (pool->disttype != disttype2str[i].type)
+		        pool_setdisttype(pool, disttype2str[i].type);
 #endif
+		      if (pool->disttype == disttype2str[i].type)
+			break;
+		    }
+		  dp = dpe;
+		}
+	      if (!(dp && *dp))
+		{
+		  pool_debug(pool, SOLV_ERROR, "testcase_read: system: could not change disttype to '%s'\n", pieces[2]);
+		  missing_features = 1;
+		}
 	    }
 	  if (strcmp(pieces[1], "unset") == 0)
 	    pool_setarch(pool, 0);
@@ -2263,11 +2749,75 @@ testcase_read(Pool *pool, FILE *fp, char *testcase, Queue *job, char **resultp,
 	  else
 	    pool_debug(pool, SOLV_ERROR, "disable: unknown package '%s'\n", pieces[2]);
 	}
+      else if (!strcmp(pieces[0], "feature"))
+	{
+	  int i, j;
+	  for (i = 1; i < npieces; i++)
+	    {
+	      for (j = 0; features[j]; j++)
+		if (!strcmp(pieces[i], features[j]))
+		  break;
+	      if (!features[j])
+		{
+		  pool_debug(pool, SOLV_ERROR, "testcase_read: missing feature '%s'\n", pieces[i]);
+		  missing_features++;
+		}
+	    }
+	  if (missing_features)
+	    break;
+	}
+      else if (!strcmp(pieces[0], "genid") && npieces > 1)
+	{
+	  Id id;
+	  /* rejoin */
+	  if (npieces > 2)
+	    {
+	      char *sp;
+	      for (sp = pieces[2]; sp < pieces[npieces - 1]; sp++)
+	        if (*sp == 0)
+	          *sp = ' ';
+	    }
+	  genid = solv_extend(genid, ngenid, 1, sizeof(*genid), 7);
+	  if (!strcmp(pieces[1], "op") && npieces > 2)
+	    {
+	      struct oplist *op;
+	      for (op = oplist; op->flags; op++)
+		if (!strncmp(pieces[2], op->opname, strlen(op->opname)))
+		  break;
+	      if (!op->flags)
+		{
+		  pool_debug(pool, SOLV_ERROR, "testcase_read: genid: unknown op '%s'\n", pieces[2]);
+		  break;
+		}
+	      if (ngenid < 2)
+		{
+		  pool_debug(pool, SOLV_ERROR, "testcase_read: genid: out of stack\n");
+		  break;
+		}
+	      ngenid -= 2;
+	      id = pool_rel2id(pool, genid[ngenid] , genid[ngenid + 1], op->flags, 1);
+	    }
+	  else if (!strcmp(pieces[1], "lit"))
+	    id = pool_str2id(pool, npieces > 2 ? pieces[2] : "", 1);
+	  else if (!strcmp(pieces[1], "null"))
+	    id = 0;
+	  else if (!strcmp(pieces[1], "dep"))
+	    id = testcase_str2dep(pool, pieces[2]);
+	  else
+	    {
+	      pool_debug(pool, SOLV_ERROR, "testcase_read: genid: unknown command '%s'\n", pieces[1]);
+	      break;
+	    }
+	  genid[ngenid++] = id;
+	}
       else
 	{
 	  pool_debug(pool, SOLV_ERROR, "testcase_read: cannot parse command '%s'\n", pieces[0]);
 	}
     }
+  while (job && ngenid > 0)
+    queue_push2(job, SOLVER_NOOP | SOLVER_SOLVABLE_PROVIDES, genid[--ngenid]);
+  genid = solv_free(genid);
   buf = solv_free(buf);
   pieces = solv_free(pieces);
   solv_free(testcasedir);
@@ -2283,11 +2833,18 @@ testcase_read(Pool *pool, FILE *fp, char *testcase, Queue *job, char **resultp,
     }
   if (closefp)
     fclose(fp);
+  if (missing_features)
+    {
+      solver_free(solv);
+      solv = 0;
+      if (resultflagsp)
+	*resultflagsp = 77;	/* hack for testsolv */
+    }
   return solv;
 }
 
 char *
-testcase_resultdiff(char *result1, char *result2)
+testcase_resultdiff(const char *result1, const char *result2)
 {
   Strqueue sq1, sq2, osq;
   char *r;
