@@ -34,9 +34,6 @@ extern "C" {
 #define SYSTEMSOLVABLE		1
 
 
-/* how many strings to maintain (round robin) */
-#define POOL_TMPSPACEBUF 16
-
 /*----------------------------------------------- */
 
 struct _Repo;
@@ -52,12 +49,20 @@ typedef struct _Datapos {
   Id dp;
 } Datapos;
 
+
+#ifdef LIBSOLV_INTERNAL
+
+/* how many strings to maintain (round robin) */
+#define POOL_TMPSPACEBUF 16
+
 struct _Pool_tmpspace {
   char *buf[POOL_TMPSPACEBUF];
   int   len[POOL_TMPSPACEBUF];
   int   n;
 };
 
+#endif
+
 struct _Pool {
   void *appdata;		/* application private pointer */
 
@@ -222,6 +227,7 @@ struct _Pool {
 #define REL_COMPAT	23
 #define REL_KIND	24	/* for filters only */
 #define REL_MULTIARCH	25	/* debian multiarch annotation */
+#define REL_ELSE	26	/* only as evr part of REL_COND */
 
 #if !defined(__GNUC__) && !defined(__attribute__)
 # define __attribute__(x)
@@ -341,6 +347,7 @@ static inline Id *pool_whatprovides_ptr(Pool *pool, Id d)
 }
 
 void pool_whatmatchesdep(Pool *pool, Id keyname, Id dep, Queue *q, int marker);
+void pool_whatcontainsdep(Pool *pool, Id keyname, Id dep, Queue *q, int marker);
 
 /* search the pool. the following filters are available:
  *   p     - search just this solvable
